/*
Copyright The Kubernetes Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

// Code generated by applyconfiguration-gen. DO NOT EDIT.

package v1

// ContainerExtendedResourceRequestApplyConfiguration represents a declarative configuration of the ContainerExtendedResourceRequest type for use
// with apply.
//
// ContainerExtendedResourceRequest has the mapping of container name,
// extended resource name to the device request name.
type ContainerExtendedResourceRequestApplyConfiguration struct {
	// The name of the container requesting resources.
	ContainerName *string `json:"containerName,omitempty"`
	// The name of the extended resource in that container which gets backed by DRA.
	ResourceName *string `json:"resourceName,omitempty"`
	// The name of the request in the special ResourceClaim which corresponds to the extended resource.
	RequestName *string `json:"requestName,omitempty"`
}

// ContainerExtendedResourceRequestApplyConfiguration constructs a declarative configuration of the ContainerExtendedResourceRequest type for use with
// apply.
func ContainerExtendedResourceRequest() *ContainerExtendedResourceRequestApplyConfiguration {
	return &ContainerExtendedResourceRequestApplyConfiguration{}
}

// WithContainerName sets the ContainerName field in the declarative configuration to the given value
// and returns the receiver, so that objects can be built by chaining "With" function invocations.
// If called multiple times, the ContainerName field is set to the value of the last call.
func (b *ContainerExtendedResourceRequestApplyConfiguration) WithContainerName(value string) *ContainerExtendedResourceRequestApplyConfiguration {
	b.ContainerName = &value
	return b
}

// WithResourceName sets the ResourceName field in the declarative configuration to the given value
// and returns the receiver, so that objects can be built by chaining "With" function invocations.
// If called multiple times, the ResourceName field is set to the value of the last call.
func (b *ContainerExtendedResourceRequestApplyConfiguration) WithResourceName(value string) *ContainerExtendedResourceRequestApplyConfiguration {
	b.ResourceName = &value
	return b
}

// WithRequestName sets the RequestName field in the declarative configuration to the given value
// and returns the receiver, so that objects can be built by chaining "With" function invocations.
// If called multiple times, the RequestName field is set to the value of the last call.
func (b *ContainerExtendedResourceRequestApplyConfiguration) WithRequestName(value string) *ContainerExtendedResourceRequestApplyConfiguration {
	b.RequestName = &value
	return b
}
